package ld27;

import java.awt.BorderLayout;
import java.awt.Canvas;
import java.awt.Dimension;
import java.io.IOException;
import java.text.DecimalFormat;
import java.util.Random;

import javax.swing.JFrame;

import ld27.entity.Player;
import ld27.entity.Zombie;
import ld27.menu.Menu;
import ld27.menu.MenuMain;
import ld27.menu.MenuScore;
import ld27.particle.Bullet;
import ld27.render.TextRenderer;
import ld27.sound.Sound;
import ld27.sound.SoundQueue;
import ld27.texture.Texture;
import ld27.texture.TileRenderer;
import ld27.world.World;

import org.lwjgl.LWJGLException;
import org.lwjgl.input.Keyboard;
import org.lwjgl.input.Mouse;
import org.lwjgl.openal.AL;
import org.lwjgl.opengl.Display;
import org.lwjgl.opengl.PixelFormat;

import static org.lwjgl.opengl.GL11.*;

public class Main extends Canvas implements Runnable {
	private static final long serialVersionUID = 1L;
	
	public static Main instance;
	
	public World world;
	public Player player;
	public Menu menu;
	
	public Sound shotgun;
	public Sound reload1;
	public Sound reload2;
	public Texture textures;
	
	public float time, spawnTime;
	public int kills;
	
	public boolean running;
	
	public Main() {
		instance = this;
		setPreferredSize(new Dimension(768, 768));
	}
	
	public static void main(String[] args) throws Exception {
		Main main = new Main();
		
		JFrame frame = new JFrame();
		frame.setLayout(new BorderLayout());
		frame.add(main);
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		frame.pack();
		frame.setVisible(true);
		frame.setResizable(false);
		
		main.start();
		main.requestFocus();
	}
	
	public void setMenu(Menu menu) {
		if(this.menu != null) {
			this.menu.destroy();
		}
		this.menu = menu;
	}
	
	public void start() {
		running = true;
		new Thread(this).start();
	}
	
	public void run() {
		try {
			Display.setParent(this);
			Display.create(new PixelFormat(1, 1, 1, 4));
		} catch(LWJGLException e) {
			e.printStackTrace();
			return;
		}
		
		glMatrixMode(GL_PROJECTION);
		float ratio = Display.getHeight() / Display.getWidth();
		glOrtho(0, 1, ratio, 0, 1, -1);
		glViewport(0, 0, Display.getWidth(), Display.getHeight());
		glMatrixMode(GL_MODELVIEW);
		glLoadIdentity();
		
		/*
		 * Load
		 */
		try {
			textures = Texture.load("textures", Texture.class.getResourceAsStream("Textures.png"));
			Texture.load("font", Texture.class.getResourceAsStream("Font.png"));
			
			shotgun = Sound.load("shotgun", Sound.class.getResourceAsStream("Shotgun.wav"), 1, 1);
			reload1 = Sound.load("reload1", Sound.class.getResourceAsStream("Reload1.wav"), 1, 1);
			reload2 = Sound.load("reload2", Sound.class.getResourceAsStream("Reload2.wav"), 1, 1);
			Sound.load("hit", Sound.class.getResourceAsStream("Hit.wav"), 1, 1);
			Sound.load("step", Sound.class.getResourceAsStream("Step.wav"), 1, 0.2f);
			Sound.load("hitzombie", Sound.class.getResourceAsStream("HitZombie.wav"), 1, 1);
			Sound.load("death", Sound.class.getResourceAsStream("Death.wav"), 1, 1);
			Sound.load("menumove", Sound.class.getResourceAsStream("MenuMove.wav"), 1, 1);
			Sound.load("menuselect", Sound.class.getResourceAsStream("MenuSelect.wav"), 1, 1);
		} catch (IOException e) {
			e.printStackTrace();
			return;
		}
		
		/*
		 * Init
		 */
		/*world = new World();
		player = new Player();
		world.add(player);*/
		menu = new MenuMain();
		
		/*
		 * GL Init
		 */
		glEnable(GL_TEXTURE_2D);
		glEnable(GL_BLEND);
		glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
		
		long lastTime = -1;
		long fpsTime = 0;
		int fps = 0;
		
		running = true;
		while(running) {
			glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
			
			handleInput();
			
			if(lastTime != -1) {
				tick((System.nanoTime() - lastTime) / 1000000000f);
				fpsTime += (System.nanoTime() - lastTime);
				while(fpsTime > 1000000000) {
					System.out.println("Fps: " + fps);
					fps = 0;
					fpsTime -= 1000000000;
				}
			}
			lastTime = System.nanoTime();
			
			draw();
			
			fps++;
			
			Display.update();
			Display.sync(120);
		}
		
		AL.destroy();
		Display.destroy();
		System.exit(0);
	}
	
	public void handleInput() {
		if(Display.wasResized()) {
			glMatrixMode(GL_PROJECTION);
			glLoadIdentity();
			float ratio = (float) Display.getHeight() / Display.getWidth();
			glOrtho(0, 1, ratio, 0, 1, -1);
			glViewport(0, 0, Display.getWidth(), Display.getHeight());
			glMatrixMode(GL_MODELVIEW);
			glLoadIdentity();
		}
		while(Keyboard.next()) {
			if(menu != null) {
				if(Keyboard.getEventKeyState()) {
					continue;
				}
				if(Keyboard.getEventKey() == Keyboard.KEY_UP) {
					menu.up();
				} else if(Keyboard.getEventKey() == Keyboard.KEY_DOWN) {
					menu.down();
				} else if(Keyboard.getEventKey() == Keyboard.KEY_RETURN) {
					menu.select();
				}
			}
		}
		while(Mouse.next()) {
			if(menu != null) {
				continue;
			}
			if(Mouse.getEventButton() == 0 && Mouse.getEventButtonState()) {
				if(player.shootTimer <= 0) {
					float angle = (float) -Math.atan2(Mouse.getY() - Display.getHeight() / 2f, Mouse.getX() - Display.getWidth() / 2f);
					Random random = new Random();
					for(int i = 0; i < 10; i++) {
						world.add(new Bullet(world, player.x, player.y, angle + (random.nextFloat() - 0.5f) * 0.4f, 50 + random.nextInt(5)));
					}
					player.shootTimer = Player.shootCooldown;
					try {
						shotgun.play();
						new SoundQueue(reload1, 600);
						new SoundQueue(reload2, 800);
					} catch(Exception e) {
						e.printStackTrace();
					}
				}
			}
			Mouse.poll();
		}
	}
	
	public void tick(float length) {
		if(menu != null) {
			return;
		}
		
		if(player.health <= 0) {
			if(player.deadTimer > 2) {
				Main.instance.setMenu(new MenuScore(kills, time));
			}
		}
		
		world.tick(length);
		
		time += length;
		spawnTime += length;
		
		while(spawnTime > 10) {
			world.add(new Zombie(world, player, 10, 10, 10));
			
			spawnTime -= 10;
		}
	}
	
	public void draw() {
		if(menu != null) {
			menu.draw();
			return;
		}
		
		TextRenderer renderer;
		
		glLoadIdentity();
		float ratio = (float) Display.getHeight() / Display.getWidth();
		glTranslatef(-player.x/16f + 0.5f, -player.y/16f + 0.5f * ratio, 0);
		
		world.draw();
		
		renderer = new TextRenderer(Texture.get("font"), timeFormat.format(10 - time % 10), 10f/16-7/256f, 10f/16-3/256f, 1f/64, 1, 1, 1, 0.5f);
		renderer.draw();
		renderer.destroy();
		
		glLoadIdentity();
		renderer = new TextRenderer(Texture.get("font"), "Survived:       " + timeFormat.format(time), 0, 0, 1f/64, 1, 1, 1, 1);
		renderer.draw();
		renderer.destroy();
		renderer = new TextRenderer(Texture.get("font"), "Zombies Killed: " + kills, 0, 2f/64, 1f/64, 1, 1, 1, 1);
		renderer.draw();
		renderer.destroy();
		
		TileRenderer heartRendererEmpty = new TileRenderer(textures, 0, 4);
		TileRenderer heartRendererHalf = new TileRenderer(textures, 1, 4);
		TileRenderer heartRendererFull = new TileRenderer(textures, 2, 4);
		
		for(int i = 0; i < 3; i++) {
			if(player.health < (i+1) * 2 - 1) {
				heartRendererEmpty.add(i, 1);
			} else if(player.health < (i+1) * 2) {
				heartRendererHalf.add(i, 1);
			} else if(player.health >= (i+1) * 2) {
				heartRendererFull.add(i, 1);
			}
		}
		
		heartRendererEmpty.draw();
		heartRendererHalf.draw();
		heartRendererFull.draw();
		
		heartRendererEmpty.destroy();
		heartRendererHalf.destroy();
		heartRendererFull.destroy();
	}
	
	public static final DecimalFormat timeFormat;
	
	static {
		timeFormat = new DecimalFormat();
		timeFormat.setMaximumFractionDigits(2);
		timeFormat.setMinimumFractionDigits(2);
	}
}
