package ld27.entity;

import static org.lwjgl.opengl.GL11.glTranslatef;

import org.lwjgl.input.Keyboard;
import org.lwjgl.input.Mouse;
import org.lwjgl.opengl.Display;

import ld27.sound.Sound;
import ld27.texture.Texture;
import ld27.texture.TileRenderer;
import static org.lwjgl.opengl.GL11.*;

public class Player extends Entity {
	public static final float speed = 0.7f;
	public static final float shootCooldown = 1.25f;
	public static final float knockbackCooldown = 0.5f;
	
	public Sound stepSound;
	
	public float walkCounter;
	public float walkSoundCounter;
	public float shootTimer;
	public float knockbackTimer;
	public float deadTimer;
	
	public int health = 6;
	public int ammo = 8;
	
	public Player() {
		x = 10;
		y = 13;
		stepSound = Sound.sounds.get("step");
	}
	
	@Override
	public void tick(float length) {
		super.tick(length);

		if(x < 0.4f) {
			x = 0.4f;
		}
		if(x > 19.6f) {
			x = 19.6f;
		}
		if(y < 0.4f) {
			y = 0.4f;
		}
		if(y > 19.6f) {
			y = 19.6f;
		}
		
		dx *= 1f - 10f * length;
		dy *= 1f - 10f * length;
		
		if(health <= 0) {
			deadTimer += length;
			shootTimer = 1;
			knockbackTimer = 1;
			return;
		}
		
		shootTimer -= length;
		knockbackTimer -= length;
		
		float mx = 0, my = 0;
		if(Keyboard.isKeyDown(Keyboard.KEY_W)) {
			my--;
		}
		if(Keyboard.isKeyDown(Keyboard.KEY_A)) {
			mx--;
		}
		if(Keyboard.isKeyDown(Keyboard.KEY_S)) {
			my++;
		}
		if(Keyboard.isKeyDown(Keyboard.KEY_D)) {
			mx++;
		}
		if(mx == 0 && my == 0) {
			return;
		}
		float angle = (float) Math.atan2(my, mx);
		dx += Math.cos(angle) * speed * length;
		dy += Math.sin(angle) * speed * length;
		walkCounter += length;
		walkSoundCounter += length;
	}

	@Override
	public void draw() {
		if(health <= 0) {
			glTranslatef(x/16f, y/16f, 0);
			
			TileRenderer renderer = new TileRenderer(Texture.get("textures"), 2, 0);
			renderer.add(-0.5f, -0.5f);
			renderer.draw();
			renderer.destroy();
			
			glTranslatef(-x/16f, -y/16f, 0);
			return;
		}
		
		float angle = (float) -Math.atan2(Mouse.getY() - Display.getHeight() / 2f, Mouse.getX() - Display.getWidth() / 2f);
		
		glTranslatef(x/16f, y/16f, 0);
		glRotatef((float) (angle / Math.PI * 180), 0, 0, 1);
		
		while(walkSoundCounter > 0.3f) {
			stepSound.play();
			walkSoundCounter -= 0.3f;
		}
		
		TileRenderer renderer = new TileRenderer(Texture.get("textures"), walkCounter % 0.5f > 0.3f ? 1 : 0, 0);
		renderer.add(-0.5f, -0.5f);
		renderer.draw();
		renderer.destroy();
		
		glRotatef((float) (-angle / Math.PI * 180), 0, 0, 1);
		glTranslatef(-x/16f, -y/16f, 0);
	}

	public void shoot() {
		
	}
}
