package ld27.entity;

import static org.lwjgl.opengl.GL11.glRotatef;
import static org.lwjgl.opengl.GL11.glTranslatef;

import java.util.Random;

import ld27.particle.Blood;
import ld27.sound.Sound;
import ld27.texture.Texture;
import ld27.texture.TileRenderer;
import ld27.world.World;

public class Zombie extends Entity {
	public static final float speed = 0.75f;
	public static final float playerKnockback = 0.4f;
	public static final float separationKnockback = 0.01f;
	
	public float walkCounter;
	public float health;
	public float deadTimer;
	
	public World world;
	public Player player;
	
	public static Sound playerHit, playerDeath;
	
	public Zombie(World world, Player player, float x, float y, float health) {
		this.world = world;
		this.player = player;
		this.x = x;
		this.y = y;
		this.health = health;
		playerHit = Sound.sounds.get("hit");
		playerDeath = Sound.sounds.get("death");
	}
	
	@Override
	public void tick(float length) {
		super.tick(length);

		if(x < 0.4f) {
			x = 0.4f;
		}
		if(x > 19.6f) {
			x = 19.6f;
		}
		if(y < 0.4f) {
			y = 0.4f;
		}
		if(y > 19.6f) {
			y = 19.6f;
		}
		
		walkCounter += length;

		dx *= 1f - 10f * length;
		dy *= 1f - 10f * length;
		
		if(health <= 0) {
			deadTimer += length;
			return;
		}
		
		float angle = (float) Math.atan2(player.y - y, player.x - x);
		
		dx += Math.cos(angle) * speed * length;
		dy += Math.sin(angle) * speed * length;
		
		for(Entity entity : world.entities) {
			if(entity == this) {
				continue;
			}
			if(Math.abs(entity.x - x) < 5 && Math.abs(entity.y - y) < 5) {
				if(distanceTo(entity) < 0.5f) {
					if(entity instanceof Player) {
						if(player.knockbackTimer > 0 || player.health <= 0) {
							continue;
						}
						
						entity.dx += Math.cos(angle) * playerKnockback;
						entity.dy += Math.sin(angle) * playerKnockback;
						
						Random random = new Random();
						float dx = (float) (Math.cos(angle) * playerKnockback);
						float dy = (float) (Math.sin(angle) * playerKnockback);
						for(int i = 0; i < 5; i++) {
							world.add(new Blood(player.x, player.y, dx + (random.nextFloat() - 0.5f) * 16f, dy + (random.nextFloat() - 0.5f) * 16f, 1));
						}
						
						player.knockbackTimer = Player.knockbackCooldown;
						player.health--;
						if(player.health == 0) {
							playerDeath.play();
							for(int i = 0; i < 20; i++) {
								world.add(new Blood(player.x, player.y, player.dx + (random.nextFloat() - 0.5f) * 32f, player.dy + (random.nextFloat() - 0.5f) * 32f, 3));
							}
						} else if(player.health > 0) {
							playerHit.play();
						}
					} else { /* Zombie */
						float zombieAngle = (float) Math.atan2(entity.y - y, entity.x - x);
						entity.dx += (float) (Math.cos(zombieAngle) * separationKnockback);
						entity.dy += (float) (Math.sin(zombieAngle) * separationKnockback);
					}
				}
			}
		}
	}

	@Override
	public void draw() {
		if(health <= 0) {
			glTranslatef(x/16f, y/16f, 0);
			
			TileRenderer renderer = new TileRenderer(Texture.get("textures"), 5, 0);
			renderer.renderer.setColor(1, 1, 1, 1 - deadTimer);
			renderer.add(-0.5f, -0.5f);
			renderer.draw();
			renderer.destroy();
			
			glTranslatef(-x/16f, -y/16f, 0);
		} else {
			float angle = (float) Math.atan2(player.y - y, player.x - x);
			
			glTranslatef(x/16f, y/16f, 0);
			glRotatef((float) (angle / Math.PI * 180), 0, 0, 1);
			
			TileRenderer renderer = new TileRenderer(Texture.get("textures"), walkCounter % 0.6f > 0.3f ? 4 : 3, 0);
			renderer.add(-0.5f, -0.5f);
			renderer.draw();
			renderer.destroy();
			
			glRotatef((float) (-angle / Math.PI * 180), 0, 0, 1);
			glTranslatef(-x/16f, -y/16f, 0);
		}
	}
}
