package ld27.particle;

import java.util.ListIterator;

import ld27.Main;
import ld27.entity.Entity;
import ld27.entity.Player;
import ld27.entity.Zombie;
import ld27.sound.Sound;
import ld27.texture.Texture;
import ld27.texture.TileRenderer;
import ld27.world.World;

public class Bullet extends Particle {
	public static final float knockback = 0.025f;

	public World world;
	
	public float angle;
	
	public static Sound hitZombie;
	
	public Bullet(World world, float x, float y, float angle, float veclocity) {
		this.world = world;
		this.x = x;
		this.y = y;
		this.angle = angle;
		this.dx = (float) (Math.cos(angle) * veclocity);
		this.dy = (float) (Math.sin(angle) * veclocity);
		this.life = 1;
		if(hitZombie == null) {
			hitZombie = Sound.sounds.get("hitzombie");
		}
	}

	public float distanceTo(Entity entity) {
		return (float) Math.sqrt(Math.pow(x - entity.x, 2) + Math.pow(y - entity.y, 2));
	}
	
	public void tick(float length, ListIterator<Particle> particles) {
		super.tick(length);
		
		if(x < 0.4f || x > 19.6f || y < 0.4f || y > 19.6f) {
			age = life;
			return;
		}
		
		dx *= 0.94f;
		dy *= 0.94f;
		
		float knockbackMultiplyer = Math.min((Math.abs(dx)+Math.abs(dy)) / 10f, 1);
		
		for(Entity entity : world.entities) {
			if(entity instanceof Player) {
				continue;
			}
			
			if(Math.abs(entity.x - x) < 5 && Math.abs(entity.y - y) < 5) {
				if(distanceTo(entity) < 0.5f) {
					entity.dx += Math.cos(angle) * knockback * knockbackMultiplyer;
					entity.dy += Math.sin(angle) * knockback * knockbackMultiplyer;
					
					hitZombie.play();
					
					if(entity instanceof Zombie) {
						((Zombie) entity).health -= 0.075f;
						if(((Zombie) entity).health+0.1f > 0) {
							if(((Zombie) entity).health <= 0) {
								Main.instance.kills++;
							}
						}
					}

					particles.add(new Blood(x, y, dx*0.1f, dy*0.1f, 2));
					age = life;
					break;
				}
			}
		}
	}
	
	@Override
	public void draw() {
		TileRenderer renderer = new TileRenderer(Texture.get("textures"), 0, 3);
		renderer.renderer.setColor(1, 1, 1, (Math.abs(dx)+Math.abs(dy)) / 10f);
		renderer.add(x + -0.5f, y + -0.5f);
		renderer.draw();
		renderer.destroy();
	}
}
