package ld27.texture;

import java.awt.image.BufferedImage;
import java.awt.image.DataBufferInt;
import java.io.IOException;
import java.io.InputStream;
import java.nio.ByteBuffer;
import java.util.HashMap;

import javax.imageio.ImageIO;

import org.lwjgl.BufferUtils;

import static org.lwjgl.opengl.GL11.*;

public class Texture {
	public static final HashMap<String, Texture> textures = new HashMap<String, Texture>();
	
	public int id;
	public int width, height;
	
	public Texture(BufferedImage image) {
		BufferedImage image2 = new BufferedImage(image.getWidth(), image.getHeight(), BufferedImage.TYPE_INT_ARGB);
		image2.getGraphics().drawImage(image, 0, 0, null);
		int[] p = ((DataBufferInt) image2.getRaster().getDataBuffer()).getData();
		byte[] p2 = new byte[p.length * 4];
		for(int i = 0; i < p.length; i++) {
			p2[i * 4] = (byte) (p[i] >> 16);
			p2[i * 4+1] = (byte) (p[i] >> 8);
			p2[i * 4+2] = (byte) p[i];
			p2[i * 4+3] = (byte) (p[i] >> 24);
		}
		width = image.getWidth();
		height = image.getHeight();
		ByteBuffer buffer = BufferUtils.createByteBuffer(width * height* 4);
		buffer.put(p2);
		buffer.flip();
		
		id = glGenTextures();
		bind();
		glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
		glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
		glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, image.getWidth(), image.getHeight(), 0, GL_RGBA, GL_UNSIGNED_BYTE, buffer);
		unbind();
	}
	
	public static Texture load(InputStream input) throws IOException {
		BufferedImage image = ImageIO.read(input);
		return new Texture(image);
	}
	
	public static Texture load(String name, InputStream input) throws IOException {
		BufferedImage image = ImageIO.read(input);
		Texture texture = new Texture(image);
		textures.put(name, texture);
		return texture;
	}
	
	public static Texture get(String name) {
		return textures.get(name);
	}
	
	public void bind() {
		glBindTexture(GL_TEXTURE_2D, id);
	}
	
	public static void unbind() {
		glBindTexture(GL_TEXTURE_2D, 0);
	}
	
	public void delete() {
		if(id != 0) {
			glDeleteTextures(id);
			id = 0;
		}
	}
}
