package ld27.world;

import java.util.ArrayList;
import java.util.ListIterator;

import ld27.Main;
import ld27.entity.Entity;
import ld27.entity.Player;
import ld27.entity.Zombie;
import ld27.particle.Bullet;
import ld27.particle.Particle;
import ld27.texture.Texture;

public class World {
	public ArrayList<Particle> particles;
	public ArrayList<Entity> entities;
	public ArrayList<Tile> tileList;
	public Tile[][][] tiles;
	
	public World() {
		particles = new ArrayList<Particle>();
		entities = new ArrayList<Entity>();
		tileList = new ArrayList<Tile>();
		tiles = new Tile[1][20][20];
		
		Texture texture = Texture.get("textures");
		
		
		Tile brickFloor = new Tile(texture, 0, 1);
		Tile brickFloorEdgeLeft = new Tile(texture, 1, 1);
		Tile brickFloorEdgeRight = new Tile(texture, 2, 1);
		Tile brickFloorEdgeTop = new Tile(texture, 3, 1);
		Tile brickFloorEdgeBottom = new Tile(texture, 4, 1);
		Tile brickFloorCornerTL = new Tile(texture, 0, 2);
		Tile brickFloorCornerTR = new Tile(texture, 1, 2);
		Tile brickFloorCornerBL = new Tile(texture, 2, 2);
		Tile brickFloorCornerBR = new Tile(texture, 3, 2);
		Tile brickFloorHoleTL = new Tile(texture, 5, 1);
		Tile brickFloorHoleTR = new Tile(texture, 6, 1);
		Tile brickFloorHoleBL = new Tile(texture, 5, 2);
		Tile brickFloorHoleBR = new Tile(texture, 6, 2);

		tileList.add(brickFloor);
		tileList.add(brickFloorEdgeLeft);
		tileList.add(brickFloorEdgeRight);
		tileList.add(brickFloorEdgeTop);
		tileList.add(brickFloorEdgeBottom);
		tileList.add(brickFloorCornerTL);
		tileList.add(brickFloorCornerTR);
		tileList.add(brickFloorCornerBL);
		tileList.add(brickFloorCornerBR);
		tileList.add(brickFloorHoleTL);
		tileList.add(brickFloorHoleTR);
		tileList.add(brickFloorHoleBL);
		tileList.add(brickFloorHoleBR);
		
		for(int x = 0; x < 20; x++) {
			for(int y = 0; y < 20; y++) {
				if(x == 0) {
					tiles[0][x][y] = brickFloorEdgeLeft;
					continue;
				}
				if(x == 19) {
					tiles[0][x][y] = brickFloorEdgeRight;
					continue;
				}
				if(y == 0) {
					tiles[0][x][y] = brickFloorEdgeTop;
					continue;
				}
				if(y == 19) {
					tiles[0][x][y] = brickFloorEdgeBottom;
					continue;
				}
				tiles[0][x][y] = brickFloor;
			}
		}
		
		tiles[0][0][0] = brickFloorCornerTL;
		tiles[0][19][0] = brickFloorCornerTR;
		tiles[0][0][19] = brickFloorCornerBL;
		tiles[0][19][19] = brickFloorCornerBR;
		
		tiles[0][9][9] = brickFloorHoleTL;
		tiles[0][10][9] = brickFloorHoleTR;
		tiles[0][9][10] = brickFloorHoleBL;
		tiles[0][10][10] = brickFloorHoleBR;
	}
	
	public void add(Entity entity) {
		entities.add(0, entity);
	}

	public void add(Particle particle) {
		synchronized(particles) {
			particles.add(particle);
		}
	}
	
	public void tick(float length) {
		for(ListIterator<Entity> it = entities.listIterator(); it.hasNext(); ) {
			Entity entity = it.next();
			if(entity instanceof Zombie) {
				if(((Zombie) entity).deadTimer >= 1) {
					it.remove();
					continue;
				}
			}
			entity.tick(length);
		}
		synchronized(particles) {
			for(ListIterator<Particle> it = particles.listIterator(); it.hasNext(); ) {
				Particle particle = it.next();
				if(particle.age >= particle.life) {
					it.remove();
				}
				if(particle instanceof Bullet) {
					((Bullet) particle).tick(length, it);
					continue;
				}
				particle.tick(length);
			}
		}
	}
	
	public void draw() {
		for(Tile tile : tileList) {
			tile.drawInit();
		}
		
		for(int x = 0; x < tiles[0].length; x++) {
			for(int y = 0; y < tiles[0][x].length; y++) {
				if(tiles[0][x][y] != null) {
					tiles[0][x][y].draw(x, y);
				}
			}
		}
		
		for(Tile tile : tileList) {
			tile.drawFinish();
		}
		
		for(Particle particle : particles) {
			particle.draw();
		}

		if(Main.instance.player.health <= 0) {
			Main.instance.player.draw();
		}
		
		for(Entity entity : entities) {
			if(entity instanceof Player) {
				if(((Player) entity).health <= 0) {
					continue;
				}
			}
			entity.draw();
		}
	}
}
